/*	
 * Component Name: EIF	
 *	
 * $Date: 8/10/00 14:52:18 $	
 *	
 * $Source:	
 *	
 * $Revision: 1.81 $	
 *	
 * Description:	
 *      TEC agent communication library for non-TME agents	
 *	
 * (C) COPYRIGHT TIVOLI Systems, Inc. 1994	
 * Unpublished Work	
 * All Rights Reserved	
 * Licensed Material - Property of TIVOLI Systems, Inc.	
 */	
	
#ifdef OS400	
   #include "QymaCpRight.h"	
   #include "qymadef.h"	
   #include "qymacnv.h"	
   #include "qymaswap.h"	
   #include <ctype.h>	
   #include <arpa/inet.h>	
   #include <sys/socket.h>	
   #include <netinet/in.h>	
#endif	
	
#include <sys/types.h>	
#include <stdio.h>	
#include <string.h>	
#include <stdlib.h>	
#include <time.h>	
#include <errno.h>	
#include <sys/stat.h>	
	
#if !defined(WIN32) && !defined(NETWARE)	
#	include <signal.h>	
#endif	
	
/*	
 * Unix only section	
 */	
#ifndef OS400	
#ifndef PC	
#	include <unistd.h>	
#endif	
	
#ifdef WIN32	
#	define strdup _strdup	
#	define sleep(_sec) Sleep((_sec) * 1000)	
#endif	
	
#ifdef OS2	
#	include <utils.h>	
	/* Next 3 lines are to get prototype of DosSleep() */	
#	include <os2/os2def.h>	
#	define INCL_DOSPROCESS yes	
#	include <os2/bse.h>	
	/* It's being defined somewhere else, but I couldn't find where. */	
#	ifdef sleep	
#		undef sleep	
#	endif	
#	define sleep(_s) DosSleep((_s) * 1000L)	
#endif	
	
#ifdef NETWARE	
#	include <nwthread.h>	
#	define sleep(_sec) delay((_sec) * 1000)	
#endif	
#endif       /* for ifndef OS400 */	

#if defined(SOLARIS2_IX86) || defined(RELIANT_UNIX)
#	include <rpc/rpcent.h>
#endif
	
#include "tec_rpc.h"	
#include "tec_eif.h"	
#include "eif_common.h"	
#include "nontme_eif.h"	
#include "slist.h"	

/*
** IY25233.1 - port IY30329
*/

extern int EIPC_VERIFY_PORTMAPPER;

	
char                    *last_msg=NULL;	
	
#ifdef OS400	
   /* prototype for tec_is_fd_available */	
   int tec_is_fd_available( eipc_handle_p_t handle );	
#endif	
	
/* Internal structure for a non-TME TEC handle */	
	
typedef struct {	
	/* Fields common to all EIF libraries */	
	tec_handle_common_t	common;	
	
	int						current;	
	
	/* TEC server IP address */	
	long						ip_addr[TECEIF_MAX_SERVERS];	
	
	/* Once a handle is created, the `port' and `ipch' may change	
	 * when we reconnect possibly to another host. */	
	unsigned short			port[TECEIF_MAX_SERVERS];	
	
	/* 1/0 depending on whether or not port comes from ServerPort config entry */	
   char						use_srvport[TECEIF_MAX_SERVERS];	
	
	/* connection-less or connection-oriented */	
	tec_delivery			type;	
	
	/* IPC handle for sending events.	
	 * In the CO mode, when the primary connection is broken, this will be	
	 * set to some secondary server, if any. NULL means no connection.  */	
	eipc_handle_p_t		ipch; 	
		
	/* number of events sent on `ipch' - used only in CO mode */	
	long						ev_count;	
} tec_handle_s_t;	
	
#ifdef OS400	
  unsigned short pmap_getport( struct sockaddr_in *psAddr, unsigned long ulPgmNum,	
                               unsigned long ulVersion, int iProtocol );	
  iconv_t iconCd = {0};                   /* EBCDIC to ASCII translation */
  static Data4Swap_t volatile d4sT = {0}; /* Swap to user */	
#endif	
	
#ifndef NETWARE	
/*	
 * Purpose: 	
 *    SIGPIPE signal handler	
 */	
	
static void	
catch_pipe(int sig)	
{	
#	ifdef PC	
		abort();	
#	else	
		signal(SIGPIPE, catch_pipe);	
#	endif	
}	
#endif /* NETWARE */	
	
/*	
 * Purpose:	
 *    Acquire the port the T/EC Server is listening to	
 *	
 * Description:	
 *      Uses portmapper to locate the server port.	
 *	
 * Returns:	
 *    Port Numver	
 *	
 * Notes/Dependencies:	
 */	
static unsigned short	
tec_get_port(long server_addr)	
{	
	
#ifndef PC	
	struct rpcent		*rpcinfo;	
#endif	
	
	struct sockaddr_in addr;	
	unsigned long		prognum;	
	unsigned short		port = 0;	
	
	/* connect to server via portmapper.  First get rcp program number	
	   being used by the TEC server */	
#if defined(PC) || defined(OS400)	
	prognum = TEC_SERVER_NUMBER;	
#else	
	rpcinfo = (struct rpcent *)NULL;	
	errno = EINTR;	
	
	while ((errno == EINTR)&&(rpcinfo == (struct rpcent *)NULL)) {	
	   errno = 0;	
	   rpcinfo = getrpcbyname(TEC_SERVER_NAME);	
	}	
	prognum = (rpcinfo == NULL) ? TEC_SERVER_NUMBER : rpcinfo->r_number;	
#endif	
	
	memset(&addr,'\0',sizeof(struct sockaddr_in));	
	
	addr.sin_family = AF_INET;	
	addr.sin_addr.s_addr = server_addr;	
	addr.sin_port = port;	
	errno = EINTR;	
	
	while ((errno == EINTR)&&(port == 0)) {	
	   errno = 0;	
#if defined(WIN32) || defined(OS400) || defined(OS2) || defined(NW4)
	   port = pmap_getport(&addr, prognum, TEC_VERSION_NUMBER, IPPROTO_TCP);	
#else
	   port = tec_pmap_getport(&addr, prognum, TEC_VERSION_NUMBER, IPPROTO_TCP);
#endif
	}	
	
	if (port == 0) {	
		tec_errno = E_SCALL;	
		return 0;	
	}	
	return(htons(port));	
}	
	
#ifndef NO_UTF8	
	
/* Utility function to prepare a utf8 message for sending. Due to the	
 * spaghetti-like nature of tec_put_event, it is necessary to have this	
 * ugly function which takes three "in-out" parameters and one "in"	
 * parameter. The alternative to this function was to have to cut and	
 * paste these lines several places in tec_put_event.	
 */	
static void 	
prepare_to_send_utf8_msg(char**new_msg, unsigned long* length,	
								 int* free_new_msg, tec_handle_common_t *common_h)	
{	
	/* save the current new_msg ptr, in case we need to free it */	
	char * tmp_msg_ptr = *new_msg;	
	
	/* reassigning new_msg ptr so that it has the magic utf8 byte prepended */ 	
	*new_msg = tec_prep_utf8_msg(common_h, *new_msg);	
	*length = (unsigned long) strlen(*new_msg);	
	
	if (*free_new_msg) {	
		/* free up the saved new_msg ptr, cuz we've reallocated it */	
		free(tmp_msg_ptr);	
	}	
	
	/* we've reallocated space for new_msg which must be freed later,	
	 * so we'll set the free_new_msg flag to TRUE */	
	*free_new_msg = TRUE;	
}	
	
#endif	
	
	
/* ------------------------------------------------------------------------- */	
	
/*	
 * Purpose:	
 *    connect to the TEC server without requiring the TME framework	
 *	
 * Description:	
 *    currently uses our IPC library.	
 *    Uses portmapper to locate the server port.	
 *    If location is null, we use the config file to determine	
 *    possibly multiple server hosts.	
 *	
 * Returns:	
 *    IPC handle	
 *	
 * Notes/Dependencies:	
 */	
	
	
#ifndef TME_AND_NON_TME	
tec_handle_t nontme_tec_create_handle_r(char*,char*,unsigned short,int,tec_delivery);	
	
tec_handle_t	
tec_create_handle_r(char *conf_file,	
                           char *location,	
                           unsigned short port,	
                           int oneway,	
                           tec_delivery type)	
{	
   return nontme_tec_create_handle_r(	
      conf_file, location, port, oneway, type);	
}	
#endif	
	
tec_handle_t	
nontme_tec_create_handle_r(char *conf_file,	
                           char *location,	
                           unsigned short port,	
                           int oneway,	
                           tec_delivery type)	
{	
	tec_handle_s_t *th;	
	struct hostent *server_host;	
	char           *env;	
	int            i;	
        int            loc=0;	
        int            max;	
        char           **ports;	
        char           chFromCode[6];	
        char           chToCode[6];	
        int            invalid_host=0;	
	int			   new_max_locations;	
	
#ifdef OS400	
#pragma exception_handler(EIF_EXP_HANDLER, 0, 0, _C2_MH_ESCAPE, _CTLA_HANDLE)	
   putenv("TISDIR=/QIBM/UserData/Tivoli/TEC/tis/cstable");	
#endif	
	
	/*	
	 * Allocate a new handle	
	 */	
	th = malloc(sizeof(*th));	
	NULL_CHECK(th);	
	th->current = -1;	
	th->ev_count = 0;	
	th->ipch = NULL;
	
	tec_errno = common_agent_handle_init(&th->common, conf_file, location, NULL);	
	
	if (tec_errno > 0) {	
		free(th);	
		tec_errno = E_HOST;	
		return(NULL);	
	}	
	
	/*	
	 * If in test mode, we are done	
	 */	
	if (th->common.test_mode)	
		return(th);	
	
	/*	
	 * Get the IP address of the server host. If the address is not in the	
	 * dotted notation, resolve the name.	
	 */	
	for (i = 0; i < th->common.max_locations; i++) {	
		th->ip_addr[i] = inet_addr(th->common.location[i]);	
	
		/*	
		 * On WinSock, INADDR_NONE is really -1	
		 */	
#ifdef OSF
		/* properly handle 64/32-bit conversion */
		if ((int)th->ip_addr[i] == -1) {	
#else
		if (th->ip_addr[i] == (long) -1) {	
#endif
	
			/* rsm: if we can't find the host on the network, we make that	
			 *      entry NULL and we'll throw it out a bit later	
			 */	
			if ((server_host = gethostbyname(th->common.location[i])) == NULL) {	
				th->common.location[i] = NULL;	
#ifdef OS400	
                                sendPgmMsg( TEC_INVALID_HOST, TEC_ADP_MSG_FILE_Q,	
                                            th->common.location[i], strlen(th->common.location[i]),	
                                            "*DIAG     " );	
                                invalid_host++;	
#endif	
                                            	
				continue;	
			}	
	
			memcpy(&th->ip_addr[i], server_host->h_addr_list[0],	
					sizeof(th->ip_addr));	
		}	
	}	
#ifdef OS400	
	/* all interps could check if we have all host names or IP address given invalid */	
	if ( invalid_host >= th->common.max_locations ) {	
		tec_errno = E_HOST;	
		free(th);	
		return(NULL);	
	}	
#endif	
	
	
	/* rsm */	
	new_max_locations = th->common.max_locations;	
	
	/* rsm: this is where we consolodate the space in the string array */	
	for (i = 0; i < th->common.max_locations; i++) {	
		if (th->common.location[i] != NULL) {	
			if (i > 0) {	
				int		k = i;	
	
				while ((th->common.location[k - 1] == NULL) && (k > 0))	
					k--;	
	
				if (th->common.location[k] == NULL) {	
					th->common.location[k] = th->common.location[i];	
					th->common.location[i] = NULL;	
				}	
	
			}	
		}	
		else {	
			new_max_locations--;	
		}	
	}	
	
	/* rsm */	
	th->common.max_locations = new_max_locations;	
	
	for (i = 0; i < th->common.max_locations; i++) {	
		th->port[i] = 0;	
		th->use_srvport[i] = 0;	
	}	
	
	if (location) loc=1;	
	        /*	
	         * Search for keyword 'ServerPort' in agent config	
	         */	
	ports = common_split_entries(&th->common, "ServerPort", &max);	
	
	for (i = loc; i < max+loc; i++) {	
	        th->port[i] = htons((unsigned short)atoi(ports[i-loc]));	
	
	                /*	
	                 * A zero port still means use the portmapper	
	                 */	
	        if (th->port[i])	
	                th->use_srvport[i] = 1;	
	}	
	
   free(ports);	
   ports = NULL;	
	
	if ( port != 0 ) {	
		/*	
		 * Do not use portmapper, since we are given a port	
		 */	
		th->port[0] = htons(port);	
		th->use_srvport[0] = 1;	
	}	
	
	/*	
	 * If not found, 0 in config file or there is more than one server	
	 * (but input `port' covers only the first) use portmapper	
	 */	
	for (i = 0; i < th->common.max_locations; i++) {	
#ifdef OS400	
		if ( (th->port[i] == 0) && (th->ip_addr[i]) )	
#else	
		if (th->port[i] == 0)	
#endif	
			th->port[i] = tec_get_port(th->ip_addr[i]);	
	}	
	
	if (type == use_default) {	
	
		/*	
		 * Use connection_less, except if CO found in config file	
		 */	
		th->type = connection_less;	
	
		if ((env = agent_getenv(th->common.agent_config, "ConnectionMode")) != NULL) {	
			if ((!strcmp(env, "co")) || (!strcmp(env, "CO"))	
				 || (!strcmp(env, "connection_oriented")))	
			{	
				th->type = connection_oriented;	
			}	
		}	
	}	
	else	
		th->type = type;	
	
	/*	
	 * If CO mode, create an IPC client	
	 */	
	if (th->type == connection_oriented) {	
		for (i = 0; i < th->common.max_locations; i++) {	
			th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i], &tec_errno);	
			if (th->ipch != NULL) {	
				th->current = i;	
				break;	
			}	
		}	
	
		/*	
		 * Set up a signal handler for SIGPIPE, for recovery from broken	
		 * connection	
		 */	
#ifndef PC	
		signal(SIGPIPE, catch_pipe);	
#endif	
	}	
	else	
		th->ipch = NULL;	
	
#ifdef OS400	
   /*************************************************************************/
   /* Initialize conversion of the Event Message from ASCII to EBCDIC       */
   /*************************************************************************/
   strcpy( chToCode, LATIN_I_CCSID);  /* Use Latin 1 (ISO 8859) code page */
   strcpy( chFromCode, DEFAULT_CCSID);               /* Use Job's CCSID  */
   if ( (env = tec_agent_getenv("FromCCSID")) != NULL ) {
       if (isdigit(*env)) {
          strncpy(chFromCode, env, sizeof(chFromCode));
       }
   }
   if ( (env = tec_agent_getenv("ServerCCSID")) != NULL ) {
       if(!strcmp(env, CALC_DEFAULT))
       {
          strcpy(chToCode, LATIN_I_CCSID);
       }
       else {
         if (isdigit(*env))
            strncpy(chToCode, env, sizeof(chToCode));
       }
   }

   /* Create handle to perform conversion */
   iconCd = QymaCodeCnvAlloc(chFromCode, chToCode);
   if ( iconCd.return_value == -1)
   {
     QymaCodeCnvDealloc(iconCd);
     sendPgmMsg(TEC_INVALID_CCSID, TEC_ADP_MSG_FILE_Q,
                chToCode, strlen(chToCode), "*DIAG     " );
     strcpy( chToCode, LATIN_I_CCSID);       /* Switch to use default CCSIDs*/
     strcpy( chFromCode, DEFAULT_CCSID);
     iconCd = QymaCodeCnvAlloc(chFromCode, chToCode);
   }
   /*************************************************************************/
   /* End of initialization of ASCII to EBCDIC conversion                   */
   /*************************************************************************/

   /* Set up the swap to user structure this is a global variable     @A1 A */	
   /*  structure                                                      @A1 A */	
   strcpy( d4sT.achFromUsr, "*CURRENT  " );                        /* @A1 A */	
   strcpy( d4sT.achToUsrPrf ,"QSECOFR   " );                       /* @A1 A */	
	
   return ((tec_handle_t)th);	
	
EIF_EXP_HANDLER:  return(NULL);            /* Exception is being handled @A1A*/	
	
#pragma disable_handler                                               /*@A1A*/	
	
#else	
	return ((tec_handle_t)th);	
#endif	
}	
	
tec_handle_common_t* nontme_get_common_part(tec_handle_t h)	
{	
   return &((tec_handle_s_t*)h)->common;	
}	
	
void nontme_tec_handle_disconnect(tec_handle_t h)	
{	
   tec_handle_s_t *th = (tec_handle_s_t*)h;	
   if (th->ipch != NULL)	
   {	
      eipc_shutdown(th->ipch);	
      eipc_destroy_handle(th->ipch);	
      th->ipch = NULL;	
   }	
}	
	
/* ------------------------------------------------------------------------- */	
	
/*	
 * Purpose:	
 *    send an event to the TEC server	
 *	
 * Description:	
 *    non-TME version	
 *	
 * Returns:	
 *    number of bytes written	
 *	
 * Notes/Dependencies:	
 */	
	
#ifndef TME_AND_NON_TME	
long nontme_tec_put_event(tec_handle_t handle, char*);	
	
long tec_put_event(tec_handle_t handle, char *message)	
{	
   return nontme_tec_put_event(handle, message);	
}	
#endif	
	
long	
nontme_tec_put_event(tec_handle_t handle, char *message)	
{	
	unsigned long  length;	
	tec_handle_s_t *th = (tec_handle_s_t *) handle;	
	char				*new_msg = message;	
	unsigned char	*non_utf8_msg = NULL;	
	int				non_utf8_msg_len = 0;	
	int				free_new_msg = 0;	
	int				destroy_handle = 0;	
	int				i;	
	int				port=0;	
	struct hostent	*server_host = NULL;	
	
#ifdef OS400	
	char *msg;                                                 /* @A1 A */	
	int  rc;                                                   /* @A1 C */	
	extern int     in_flush_events;          /* defined in evt_buffer.c */	
	static int swap_fail = 0;                                  /* @A1 A */	
	
	#pragma exception_handler(EIF_EXP_HANDLER, 0, 0, _C2_MH_ESCAPE,  \	
	                         _CTLA_HANDLE)                     /* @A1 A */	
#endif	
	
	/* rsm: variables used to figure out the size of the buffer */	
	long                buffer_size;	
#ifndef NO_UTF8	
	char *orig_msg_ptr = NULL;	
	
 	/* if we're flushing events, should not convert, since events have been 	
 	 * converted before they were cached. */	
	/* if NO_UTF8_CONVERSION is defined in tec_forward.conf, should not convert.
	   the events being forwarded have been converted already. */

	if(!th->common.in_flush_events && !th->common.no_utf8_conversion) {	
		/* IY24129 make a copy of the message before it is converted to utf8 for 
		 * the call to common_filter_event.
		 */
		non_utf8_msg = strdup(message);
		NULL_CHECK(non_utf8_msg);	

		/* convert the incoming message to utf8 */	
		message = tec_to_utf8(message, th->common.adapter_local_cs_handle);	
	
		/* need to save the ptr, cuz we've already allocated new memory in 	
	 	* the tec_to_utf8() function call which will need to be freed; if	
	 	* we don't save this ptr, we'll have a potential memory leak (e.g.	
	 	* if/when we later call tec_normalize_msg and reassign the ptr).	
	 	*/	
		orig_msg_ptr = message;	
	}	

	/* IY24129 If the message comes into tec_put_event in UTF8 format, we need 
	 * to convert it back out of UTF8 format so that when it goes through 
	 * common_filter_event, the delimeters are found correctly (that function 
	 * assumes the event is not in utf8 format). 
	 */
	if(!th->common.in_flush_events && th->common.no_utf8_conversion) {
		non_utf8_msg_len = strlen(message) * TIS_FROM_UTF8_RATIO + 1;
		non_utf8_msg = (unsigned char *)malloc(non_utf8_msg_len);
		NULL_CHECK(non_utf8_msg);	
		tis_from_utf8(th->common.adapter_local_cs_handle, (unsigned char *) message,
						  -1, non_utf8_msg, non_utf8_msg_len);
	}
#endif	
	
	if (NULL != message)	
		length = (unsigned long) strlen(message);	
	else	
		length = 0;	
	
	/*	
	 * This stuff was already done before this event was put into the buffer	
	 */	
	if (!th || !th->common.in_flush_events) {	
	
		/*	
		 * Make sure the event has the right termination	
		 */	
		if ((new_msg = tec_normalize_msg(message, length)) != NULL) {	
			free_new_msg = 1;	
			length = (unsigned long) strlen(new_msg);	
		}	
		else	
			new_msg = message;	
	
		/*	
		 * Check if event should be discarded due to filtering	
		 */	
		if (!th->common.dont_filter	
			 && (common_filter_event(&th->common, non_utf8_msg, FILTER_EVENT_TYPE)	
				  == FILTER_DISCARD))	
		{	
			if (free_new_msg)	
				free(new_msg);	
	
#ifndef NO_UTF8	
			/* NOTE: This function has way, way too	
			 * many exit pts (places where it returns).	
			 * This makes it very ugly, cuz we have to	
			 * do this memory cleanup in many places.	
			 * Also, if anyone should add a return in	
			 * the future, they must do this cleanup	
			 * prior to the return to avoid a memory leak.	
			 * Maybe someone should reorganize this entire	
			 * function some day (over the rainbow)?	
			 */	
	
			/* free up memory before returning */	
			if (NULL != orig_msg_ptr) {	
				free(orig_msg_ptr);	
				orig_msg_ptr = NULL;	
			}	
#endif	
			free(non_utf8_msg);
			return(0);	
		}	
	}	
	
/* APAR IY46800 - Initially prepare the msg, and only prepare once */
#ifndef NO_UTF8
    prepare_to_send_utf8_msg(&new_msg, &length, &free_new_msg, &(th->common));
#endif

	/*	
	 * Create a handle if necessary	
	 */	
	if (th == NULL) {	
		th = (tec_handle_s_t *) tec_create_handle(NULL, 0, 0, connection_less);	
		destroy_handle = 1;	
	}	
	
	/*	
	 * If we are processing a real event, assume the connection is ok.  If it's	
	 * not, this will be set to TRUE at the end right before calling	
	 * common_buffer_event()	
	 */	
	if (length != 0)	
		TECAD_no_connection = FALSE;	
	
	/*	
	 * if we are in test mode, then just dump the stuff to the output file and	
	 * return	
	 */	
	if (th->common.test_mode) {	
   /* 154975 - Do not convert to utf8 before sending the text to the
    * TestMode file. On the AS/400, the UTF8 ebcdic data will look like garbage.
    */
#ifndef OS400
		fwrite(new_msg, sizeof (char), length, th->common.test_out);	
#else
      length = strlen(non_utf8_msg);
      fwrite(non_utf8_msg, sizeof (char), length, th->common.test_out);
#endif
		putc('\n', th->common.test_out);	
		fflush(th->common.test_out);	
      free(non_utf8_msg);
	
		if (free_new_msg)	
			free(new_msg);	
	
		if (destroy_handle)	
			nontme_tec_destroy_handle((tec_handle_t) th);	
	
#ifndef NO_UTF8	
		/* free up memory before returning */	
		if (NULL != orig_msg_ptr) {	
			free(orig_msg_ptr);	
			orig_msg_ptr = NULL;	
		}	
#endif	
		return(length);	
	}	
	
	if (!th->common.in_flush_events)	
	{	
      buffer_size = tec_event_buffer_size(th, NULL, NULL);	
      	
      if (buffer_size < 0) 	
      {	
		    /* This probably means that the buffer doesn't exist,	
		     *	and it will be created later on, so let's consider	
		     *	it a buffer of size 0 */	
	
		    buffer_size = 0;	
		}	
	}	
	
	if (th->type == connection_oriented) {	
		if (th->ipch != NULL &&	
		   (th->current == 0 || th->ev_count < MAX_EVENTS_ON_SCHANNEL))	
		{	
#ifdef OS400	
			if ( tec_is_fd_available( th->ipch ) ) {
#endif	
			int rc=0;	
	
			/* rsm: Check if the cache file is bigger than 0.  If not,	
			   we won't bother to do anything except send the new event in */	
			if ((buffer_size > 0) && (!th->common.in_flush_events)) {	
				/* rsm: Try to send a NULL event.  If it succeeds,	
				   then flush any buffered events */	
				/*89364 rc = eipc_send(th->ipch, "", 0, 0, 0, &tec_errno);*/	
	
	
				if (rc == 0) {	
#ifdef OS400	
				    QymaSwapToUsr( &d4sT );	
				    tec_flush_events(handle);	
				    QymaSwapBack( &d4sT );	
#else	
				    tec_flush_events(handle);	
#endif	
				}	
			}	
	
			/* IY27890 - Before trying to send the event, call tec_get_port to see if
			 * the connection is up. If port comes back 0, the connection is down.
			 * IY25233.1 - Add EIPC_VERIFY_PORTMAPPER flag check (note, this fix should
			 * also fix the postemsg problem, where postemsg is automatically buffering events).
			 */
			if (EIPC_VERIFY_PORTMAPPER) {
				port = tec_get_port(th->ip_addr[0]);
				if (port != 0) {
					/* Now send the new event */	
					rc = eipc_send(th->ipch, new_msg, length, 0, 0, &tec_errno);	
					if (rc == 0) {	
						th->ev_count++;	
	
						if (last_msg) free(last_msg);	
						last_msg=(char*)malloc(length+1);	
						NULL_CHECK(last_msg);	
						strcpy(last_msg,new_msg);	
	
		      		if (free_new_msg)	
							free(new_msg);	
	
						if (destroy_handle)	
							nontme_tec_destroy_handle((tec_handle_t) th);	
	
#ifndef NO_UTF8	
						/* free up memory before returning */	
						if (NULL != orig_msg_ptr) {	
							free(orig_msg_ptr);	
							orig_msg_ptr = NULL;	
						}	
#endif	
						if (non_utf8_msg)
							free(non_utf8_msg);
						return(length);	
					} /* if rc == 0 */	
				} /* if (port != 0) */
			} /* if EIPC_VERIFY_PORTMAPPER */
			else {
				/* Now send the new event
				 * rc = 0 is a successful return code.
				 */
				if(th->ipch)                                 /*JHC 146491*/
				{                                            /*JHC 146491*/
					rc = eipc_send(th->ipch, new_msg, length, 0, 0, &tec_errno);	
					if (rc == 0) {
						th->ev_count++;	

						if (last_msg) free(last_msg);	
						last_msg=(char*)malloc(length+1);	
						NULL_CHECK(last_msg);	
						strcpy(last_msg,new_msg);	

						if (free_new_msg)	
							free(new_msg);	

						if (destroy_handle)	
							nontme_tec_destroy_handle((tec_handle_t) th);	

#ifndef NO_UTF8	
						/* free up memory before returning */	
						if (NULL != orig_msg_ptr) {	
							free(orig_msg_ptr);	
							orig_msg_ptr = NULL;	
						}	
#endif	
						if (non_utf8_msg)
							free(non_utf8_msg);
						return(length);	
					} /* if rc == 0 */	
				}                                           /*JHC 146491*/
			} /* else of if EIPC_VERIFY_PORTMAPPER */

#ifdef OS400	
			} /* end  if tec_is_fd_available() */
#endif	
		}	
	
		if (th->ipch) {	
			/* The old connection is broken. We wait for the retry_window	
				in hope of getting back a connection to the same host or	
				possibly a more primary server before switching to a secondary.	
				This takes care of quick server restarts to load a new rulebase	
				for example.	
				*/	
	
         eipc_destroy_handle(th->ipch);	
         th->ipch = NULL;	
         th->current = -1;	
         th->ev_count = 0;	
	
         /* WORKAROUND - The TEC gateway needs to handle retries at a higher	
          *   so we have to disable retry handling here.	
          */	
         if (gw_hack_running_in_gateway) {
	    if (non_utf8_msg)
		free(non_utf8_msg);
            return -1;	
	 }
         else	
         {	
#ifdef OS400	
            /* don't need to worry about alarms or signals in OS/400 */	
            usleep(th->common.retry_window);	
#else	
            long wait_until = time(NULL) + th->common.retry_window;	
	
            /* We cannot just sleep on retry_window since alarms or	
               other signals will wake us up.	
				*/	
            while (time(NULL) < wait_until)	
               sleep(5);	
#endif	
         }	
		}	
	
		/* try another host/port; here we get a new port immediately and not	
			try to reuse the old one first (unlike connectionless mode). */	
		for (i = 0; i < th->common.max_locations; i++) {	
			if (th->use_srvport[i] == 0) {	
				th->port[i] = tec_get_port(th->ip_addr[i]);	
			}	
			if (th->port[i] != 0) {	
				th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i],	
								 &tec_errno);	
			}	
	
			/* If we can't create the ipc connection, re-resolve name to IP address,	
			 * in case host's IP address has changed. If the ipc connection still	
			 * can't be established after re-resolving the name to an address, then	
			 * we will move on to the next TEC server in the conf file, etc.	
			 */	
			if (th->ipch == NULL) {	
	
				th->ip_addr[i] = inet_addr(th->common.location[i]);	
#ifdef OSF
				/* properly handle 64/32-bit conversion */
				if ((int)th->ip_addr[i] == -1) {	
#else
				if (th->ip_addr[i] == (long) -1) {	
#endif
					server_host = gethostbyname(th->common.location[i]);	
					if (server_host != NULL) {	
						memcpy(&th->ip_addr[i], server_host->h_addr_list[0], sizeof(th->ip_addr[0]));	
						if (th->use_srvport[i] == 0) {	
							th->port[i] = tec_get_port(th->ip_addr[i]);	
						}	
 	      			if (th->port[i] != 0) {	
							th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i], &tec_errno);	
 		     			}	
#ifdef DEBUG	
			  			printf("re-resolved host name to IP address connection-oriented\n");	
			  			printf("host name is: %s \n", th->common.location[i]);	
		  				printf("new ip address is: %s \n",	
										inet_ntoa(*(struct in_addr*)(server_host->h_addr_list[0])));	
#endif	
					}	
				}	
#ifdef DEBUG	
				else {	
			  		printf("host address was provided in numerical format - connection oriented\n");	
					printf("host name is: %s \n", th->common.location[i]);	
				}	
#endif	
		  }	
	
			if (th->ipch != NULL) {	
	
				int rc=0;	
	
				/* if the send succeeds, then flush any buffered	
				events and return */	
	    if ((buffer_size > 0) && (!th->common.in_flush_events)) {	
	    	/* rsm: Try to send a NULL event.  If it succeeds,	
					 * then flush any buffered events */	
	        /*89364 rc = eipc_send(th->ipch, "", 0 , 0, 0, &tec_errno);*/	
	
	       if (rc == 0) {	
#ifdef OS400	
		QymaSwapToUsr( &d4sT );	
	       	tec_flush_events(handle);	
		QymaSwapBack( &d4sT );	
#else	
	       	tec_flush_events(handle);	
#endif	
	        }	
	    }	
	
	    /* Now send the new event */	
	    rc = eipc_send(th->ipch, new_msg, length, 0, 0, &tec_errno);	
	    if (rc == 0) {	
	
					th->current = i;	
					th->ev_count++;	
	
					if (last_msg) free(last_msg);	
					last_msg=(char*)malloc(length+1);	
					NULL_CHECK(last_msg);	
					strcpy(last_msg,new_msg);	
	
	            if (free_new_msg)	
  		           	free(new_msg);	
	
					if (destroy_handle)	
						nontme_tec_destroy_handle((tec_handle_t) th);	
	
	
#ifndef NO_UTF8	
					/* free up memory before returning */	
					if (NULL != orig_msg_ptr) {	
						free(orig_msg_ptr);	
						orig_msg_ptr = NULL;	
					}	
#endif	
					if (non_utf8_msg)
						free(non_utf8_msg);
					return(length);	
				}	
				else {	
					eipc_destroy_handle(th->ipch);	
					th->ipch = NULL;	
					th->current = -1;	
					th->ev_count = 0;	
				}	
			}	
		}	
	
	}	
	else {	
		/*	
		 * We are connectionless, so create a new connection to the server.	
		 * If the connection fails, it could have been because the server was	
		 * restarted and the old port information is bogus, so try to get a	
		 * new port, and try to get a connection again	
		 */	
		for (i = 0; i < th->common.max_locations; i++) {	
			/*	
			 * try the old port first, more than likely the port is the same.	
			 * Contacting portmapper on every put would be expensive.	
			 */	
			if (th->ipch == NULL) {  /* only create if needed */	
			   th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i], &tec_errno);	
			}	
			if (th->ipch == NULL) {	
			  if (th->use_srvport[i] == 0) {	
				 th->port[i] = tec_get_port(th->ip_addr[i]);	
			  }	
			  if (th->port[i] != 0) {	
				 th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i], &tec_errno);	
			  }	
			}	
	
			/* If we can't create the ipc connection, re-resolve name to IP address,	
			 * in case host's IP address has changed. If the ipc connection still	
			 * can't be established after re-resolving the name to an address, then	
			 * we will move on to the next TEC server in the conf file, etc.	
			 */	
			if (th->ipch == NULL) {	
				th->ip_addr[i] = inet_addr(th->common.location[i]);	
#ifdef OSF
				/* properly handle 64/32-bit conversion */
				if ((int)th->ip_addr[i] == -1) {	
#else
				if (th->ip_addr[i] == (long) -1) {	
#endif
					server_host = gethostbyname(th->common.location[i]);	
					if (server_host != NULL) {	
						memcpy(&th->ip_addr[i], server_host->h_addr_list[0], sizeof(th->ip_addr[0]));	
						if (th->use_srvport[i] == 0) {	
							th->port[i] = tec_get_port(th->ip_addr[i]);	
						}	
       				if (th->port[i] != 0) {	
							th->ipch = eipc_create_remote_client(th->ip_addr[i], th->port[i], &tec_errno);	
      				}	
#ifdef DEBUG	
			  			printf("re-resolved host name to IP address - connectionless\n");	
			 			printf("host name is: %s \n", th->common.location[i]);	
		 		 		printf("new ip address is: %s \n",	
										inet_ntoa(*(struct in_addr*)(server_host->h_addr_list[0])));	
#endif	
					}	
				}	
#ifdef DEBUG	
				else {	
			  		printf("host address was provided in numerical format - connectionless \n");	
					printf("host name is: %s \n", th->common.location[i]);	
				}	
#endif	
			  }	
	
			/* if we finally got a connection, then send the data */	
			if (th->ipch != NULL) {	
	
			    int rc=0;	
	
			    if ((buffer_size > 0) && (!th->common.in_flush_events)) {	
					/* rsm: we have a buffer to work with and we're	
				 	* not currently flushing the buffer through the	
				 	* recursive call from tec_flush_events()	
					*/	
	
					/* rsm: send a NULL event in order to test for	
					 * the viability of the connection */	
					/*89364 rc = eipc_send(th->ipch, "", 0, 0, 0, &tec_errno);*/	
	
#if 0	
					eipc_shutdown(th->ipch);	
					eipc_destroy_handle(th->ipch);	
					th->ipch = NULL;	
#endif	
	
					/* if the send succeeded, then flush any buffered events */	
					if (rc == 0) {	
						int sent;	
						sent = tec_flush_events(handle);	
					}	
			   }	
	
			if (th->ipch == NULL)	
			{	
			   /* rsm: Send our new event now that the buffer's	
					     * been sent successfully */	
			   for (i = 0; i < th->common.max_locations; i++) {	
					/* try the old port first, more than likely the port is	
				 	* the same. Contacting portmapper on every put would be	
				 	* expensive.	
				 	*/	
	
					th->ipch = eipc_create_remote_client(th->ip_addr[i],	
								     th->port[i], &tec_errno);	
					if (th->ipch == NULL) {	
				   	 if (th->use_srvport[i] == 0) {	
							th->port[i] = tec_get_port(th->ip_addr[i]);	
				   	 }	
	
				    	if (th->port[i] != 0) {	
							th->ipch = eipc_create_remote_client(th->ip_addr[i],	
									     th->port[i], &tec_errno);	
				  		}	
					}	
	
					/* rsm: my only concern here is that the return	
					 * value from eipc_send isn't checked before new_msg	
					 * is freed and gotten rid of -- could this be	
					 * a problem? */	
					if (th->ipch != NULL) {	
				    	rc =  eipc_send(th->ipch, new_msg, length, 0, 0, &tec_errno);	
                  if (rc != 0 || !th->common.in_flush_events)	
                  {	
                     eipc_shutdown(th->ipch);	
                     eipc_destroy_handle(th->ipch);	
                     th->ipch = NULL;	
                  }	
	
                  if (rc == 0)	
                  {	
                     if (free_new_msg)	
                        free(new_msg);	
	
                     if (destroy_handle)	
                        nontme_tec_destroy_handle((tec_handle_t) th);	
	
                     break;	
                  }	
               }	
            }	
			}	
			else	
			{	
				rc =  eipc_send(th->ipch, new_msg, length, 0, 0, &tec_errno);	
				if (rc != 0 || !th->common.in_flush_events)	
				{	
					eipc_shutdown(th->ipch);	
					eipc_destroy_handle(th->ipch);	
					th->ipch = NULL;	
				}	
	
            if (rc == 0)	
            {	
					if (free_new_msg)	
						free(new_msg);	
					if (destroy_handle)	
						nontme_tec_destroy_handle((tec_handle_t) th);	
            }	
			}	
	
         if (rc == 0)	
         {	
#ifndef NO_UTF8	
				/* free up memory before returning */	
				if (NULL != orig_msg_ptr) {	
					free(orig_msg_ptr);	
					orig_msg_ptr = NULL;	
				}	
#endif	
	    if (non_utf8_msg)
		free(non_utf8_msg);
            return(length);	
         }	
         } /* matches th->ipch != NULL test */	
	
		}	
	}	
	
	/*	
	 * if we got here, then we have not sent the message to the Event Server,	
	 * so we buffer it for sending later	
	 */	
#ifdef OS400	
	QymaSwapToUsr( &d4sT );  /* swap user to QSECOFR */	
#endif	
	
	/*	
	 * failed to create a connection for a real message, set to TRUE	
	 */	
	if (length != 0)	
	   TECAD_no_connection = TRUE;	
	
	/* The AS/400 checks the port before doing an actual send, so it does 
	 * not need this last_msg logic to know if a message has gone through
	 * or not.
	 * The check for '\x03' is checking for the magic byte on the front 
	 * of the msg. All messages should have that byte on the front of them.
	 * If it is not there, it means the msg has gone through the function
	 * prepare_to_send_utf8 (which pulls the magic byte if the 
	 * Pre37Server Flag = YES). ref defect 104424.
	 */
#ifndef OS400
   if (last_msg) {	
#ifndef NO_UTF8	
		if ( last_msg[0] != '\x03' )
			last_msg = tec_to_utf8(last_msg, th->common.adapter_local_cs_handle);
#endif
		i = common_buffer_event(&th->common, last_msg);	
		free(last_msg);	
		last_msg=NULL;	
   }	
#endif
	
	i = common_buffer_event(&th->common, new_msg);	
	
#ifdef OS400	
	QymaSwapBack( &d4sT );     /* swap back no matter what happened */	
#endif	
	
	if (free_new_msg)	
		free(new_msg);	
	
	if (destroy_handle)	
		nontme_tec_destroy_handle((tec_handle_t) th);	
	
#ifndef NO_UTF8	
	/* free up memory before returning */	
	if (NULL != orig_msg_ptr) {	
		free(orig_msg_ptr);	
		orig_msg_ptr = NULL;	
	}	
#endif	
	if (non_utf8_msg)
		free(non_utf8_msg);
	return(i);	

	/* If the AS/400 gets an exception, return a -1 here. */
#ifdef OS400
	EIF_EXP_HANDLER:
		return(-1);

	#pragma disable_handler
#endif

}	
	
/*	
 * This is defined here so that the EXTERN_DECL from tec_eif.h isn't undefined	
 */	
long	
tec_put_event_to_location(tec_handle_t handle, char *message) 	
{	
	return(0);	
}	
	
/* ------------------------------------------------------------------------- */	
	
/*	
 * Purpose:	
 *    destroy TEC handle - non-TME version	
 *	
 * Description:	
 *	
 * Returns:	
 *	
 * Notes/Dependencies:	
 */	
	
#ifndef TME_AND_NON_TME	
void nontme_tec_destroy_handle(tec_handle_t);	
	
void tec_destroy_handle(tec_handle_t handle)	
{	
   nontme_tec_destroy_handle(handle);	
}	
#endif	
	
void	
nontme_tec_destroy_handle(tec_handle_t handle)	
{	
	tec_handle_s_t *th;	
#ifdef OS400	
	#pragma exception_handler(EIF_EXP_HANDLER, 0, 0, _C2_MH_ESCAPE,  \	
				  _CTLA_HANDLE)                /* @A1 A */	
#endif	
	th = (tec_handle_s_t *)handle;	
	
	if (th->type == connection_oriented) {	
		if (th->ipch)  {	
			eipc_shutdown(th->ipch);	
			eipc_destroy_handle(th->ipch);	
		}	
	}	
	
#ifndef NO_UTF8	
	tec_utf8_cleanup(&(th->common));	
#endif	
	
   /* Close event cache if it's open. */	
   cache_destructor(&th->common.event_cache);	
	
	/* only the first location was allocated and divided up */	
	free(th->common.location[0]);	
	free(th);	
	
#ifdef OS400	
        /* Destroy EBCDIC to ASCII conversion handle */
        QymaCodeCnvDealloc(iconCd);

	/* Swap the user back just in case we are being called from a cancel handler */	
	QymaSwapBack( &d4sT);                                     /* @A1A */	
	QymaRlsPrf( &d4sT );                                      /* @A1A */	
	
EIF_EXP_HANDLER:                       /* Exception is being handled @A1A */	
	
#pragma disable_handler                                           /* @A1A */	
#endif	
        return;	
	
}	
	
int	
nontme_tec_connection_status(tec_handle_t t_handle)	
{	
	tec_handle_s_t *th = (tec_handle_s_t *) t_handle;	
	
	return(((th->type == connection_less) || (th->ipch != NULL)) ? 1 : 0);	
}	
	
/* ------------------------------------------------------------------------- */	
	
/*	
 * Purpose:	
 *    initialization for non-TME TEC agent	
 *	
 * Description:	
 *	
 * Returns:	
 *	
 * Notes/Dependencies:	
 */	
	
#ifndef TME_AND_NON_TME	
int	
tec_agent_init(char *cfgfile)	
{	
#ifdef OS400	
#pragma exception_handler(EIF_EXP_HANDLER, 0, 0, _C2_MH_ESCAPE,  \	
                            _CTLA_HANDLE)               /* @A1A*/	
#endif	
	
	return(common_agent_init(cfgfile, NULL));	
	
#ifdef OS400	
 EIF_EXP_HANDLER: return(1);    /* Exception is being handled @A1A*/	
	
#pragma disable_handler                                     /*@A1A*/	
#endif	
}	
	
int	
tec_method_agent_init(char *cfgfile)	
{	
	return(common_agent_init(cfgfile, NULL));	
}	
#endif /* TME_AND_NON_TME */	
	
#ifdef OS400	
	
/*	
 * Purpose:	
 *    Checks if the incoming file descriptor is ready to receive data.	
 *	
 * Description:	
 *    Do a select on the fd and wait up to 3 seconds to see if it's ready.	
 *	
 * Returns:	
 *    TRUE  = it's ready to be written to.	
 *    FALSE = it's not ready.	
 *	
 * Notes/Dependencies: Fix for IX89688,	
 */	
	
int tec_is_fd_available( eipc_handle_p_t handle )	
{	
   fd_set                fdvar;        /* fd set                      */	
   int                   maxfd=1;      /* Max fd's to check           */	
   int                   iRc;          /* return code                 */	
   struct timeval        swait_time;   /* seconds to wait             */	
	
	
   FD_SET (handle->fd, &fdvar);    /* set the bit */	
   maxfd = handle->fd;             /* max number to check */	
   ++maxfd;	
	
   swait_time.tv_sec = 3;          /* Wait max of 3 secs for select to return */	
   iRc = select ( maxfd, (fd_set *)0, &fdvar, (fd_set *)0, &swait_time);	
   if ( iRc > 0 )	
      iRc = FD_ISSET(handle->fd, &fdvar);	

   /* IY15145
    * Send an empty event, then sleep for 1 second, this is the only way 
    * to cause the event sent after this to fail.
    */
   eipc_send(handle, "\x03", 1, 0, 0, &tec_errno);
   usleep(1);
	
   return( iRc > 0 ? 1 : 0);	
}	
	
#endif	
	
	
/*	
   These 4 functions (tec_is_secure/wrapper_thread_delay/wrapper_timeout/
							 wrapper_sleep)	
   have been created in both libtec.a & libteceif.a so they can be called	
   from liblogfile.a.  It is not until the Logfile Adapter is linked	
   that we know if we will be secure or unsecure, so these functions are	
   needed in both libraries or else the unsecure adapter won't compile.	
	
   10/6/97 - Sean Starke	
*/	
	
#ifndef TME_AND_NON_TME	
/* Are we secure or unsecure? */	
int	
tec_is_secure()	
{	
  return FALSE;	
}	
	
/* Wrapper function for tmf_thread_delay().  This will not be called	
   when running the unsecure adapter, but must exist so it will compile.	
   When secure adapter calls it the arguments are just passed on. */	
int	
wrapper_thread_delay(struct timeval *timespan)	
{	
	return 0;	
}	
	
/* Wrapper function for tmf_timeout().  This will not be called	
   when running the unsecure adapter, but must exist so it will compile.	
   When secure adapter calls it the arguments are just passed on. */	
int	
wrapper_timeout(void (*func)(), void *data, struct timeval *timer)	
{	
	return 0;	
}	

/*
** Wrapper function for sleep/tmf_thread_sleep.  'sleep' should not
** be called from TME code because some TME binaries call tmf_timeout
** and 'sleep' interferes with FW timers.  However, NON-TME code
** obviously cannot call tmf_thread_sleep because it is provided by FW.    
*/
void wrapper_sleep(int seconds)
{
#ifndef OS400
	sleep(seconds);
#else
	usleep(seconds);
#endif
}
#endif /* TME_AND_NON_TME */	
