/*
 * Component Name:  EIF
 *
 * $Date: 7/15/02 16:32:18 $
 *
 * $Source: src/eif/common/eif_common.h, tec_eif, tec_3.8 $
 *
 * $Revision: 1.3.1.19 $
 *
 * Dependencies:
 *
 * Description:
 *    Common header file for all versions of EIF:
 *       Managed Node - tme_agent_comm.c
 *       Endpoint     - lcf_agent_comm.c
 *       non-TME      - agent_comm.c
 *
 *    This header file is not delivered with the EIF product, so it should
 *    not include any definitions or prototypes needed in the public
 *    interface.  It is part of the "EIF Source Kit", though, which means
 *    that it may be delivered for porting to other platforms.  Therefore,
 *    dependencies on other headers and libraries (e.g., TMF) should be
 *    minimized.
 *
 * (C) COPYRIGHT TIVOLI Systems 1997
 * Unpublished Work
 * All Rights Reserved
 * Licensed Material - Property of TIVOLI Systems
 */

#ifndef eif_common_h
#define eif_common_h

#include "slist.h"
#include "tec_defines.h"
#include "tec_eif.h"

#ifndef NO_UTF8
	#if defined(ENDPOINT_BUILD) || defined(ADAPTERS_ONLY_UNIX) || defined(NW4) || defined(OS400)
		#include <mrt/tis.h>
	#else
		#include <tivoli/tis.h>
	#endif
#endif

#define TECEIF_MAX_SERVERS	8
#define MAX_EVENTS_ON_SCHANNEL	64

typedef unsigned char	eif_boolean;

/*
 * Structure for event buffer configuration information.
 */
typedef struct {
	char				*buffer_path;
	unsigned long	max_size;
	unsigned long	shrink_size;    /* Ignored. */
	unsigned long	read_blk_len;   /* Ignored. */
	unsigned long	event_max_size;
	int				flush_rate;
	eif_boolean		initialized;
	eif_boolean		do_buffer;
   unsigned long  maxPacketSize;  /* def 126118 max size in bytes of packet */
} BufferConfig;

/*
 * Structure for EIF configuration information.
 */
typedef struct {
	BufferConfig	buffer_info;
	SL_t				*tec_env_list;
	SL_t				*tec_pre_filters;
	SL_t				*tec_event_filters;
	SL_t				*tec_event_cache_filters;
	int				filter_in;
	int				pre_filter_in;
} AgentConfig;

/* ** EVENT CACHE **
 * Structure and functions keeping track of the state of the event cache file
 * associated each type of EIF (Managed Node, Endpoint, non-TME).  You should
 * only use the functions declared below to manipulate the cache file and
 * the tec_event_cache_t structure.  The functions are defined in
 * 'evt_buffer.c'.
 */
typedef struct {
   char          *file_path;     /* Name of the cache file. */
   HANDLE         fd;            /* Handle to open cache file. */
   long           maxsz;         /* Maximum size of the cache file in bytes. */
   long           head;          /* Offset to next event to snd in cache file */
   long           tail;          /* Offset to 1st free byte in cache file. */
   long           tail_on_open;  /* Value of tail when the cache file was
                                  * opened. */
   long           head_at_read;  /* Position of head when we last read data.*/
   long           bytes_read;    /* The size of the chunk of data we read. */
   char          *buf;           /* Buffer of chunk of data from cache file. */
   char          *curr_pos;      /* Current position in buf. */
   long           curr_step_no;  /* Supports the fuction cache_curr_step_no()
                                  * in ../eif/evt_buffer.c. */
   long           tail_to_write; /* Tail position to write bytes buffered in
                                  * wbuf. */
   long           bytes_to_write;/* Number of bytes in wbuf. */
   char          *wbuf;          /* Buffer of bytes that need to be written
                                  * to disk. */
   int            read_only;     /* In read only mode?  Used during event
                                  * cache dump. */
} tec_event_cache_t;

/* Enum identifiers used w/ cache_dump(). */
typedef enum {CACHE_DUMP_ALL, CACHE_DUMP_QUEUED}
        cache_dump_mode_t;

/* Callback function type used w/ cache_dump(). */
typedef eif_boolean (*cache_dump_func_t)(char* const, eif_boolean, void*);

EXTERN_DECL(void cache_constructor, (tec_event_cache_t *));
EXTERN_DECL(void cache_destructor, (tec_event_cache_t *));
EXTERN_DECL(int cache_resize_file, (tec_event_cache_t *, long));
EXTERN_DECL(eif_boolean cache_open_file,
            (tec_event_cache_t *, const char *, long));
EXTERN_DECL(eif_boolean cache_is_open, (const tec_event_cache_t *));
EXTERN_DECL(const char* cache_file_name, (const tec_event_cache_t *));
EXTERN_DECL(long cache_max_size, (const tec_event_cache_t *));
EXTERN_DECL(eif_boolean cache_is_empty, (const tec_event_cache_t *));
EXTERN_DECL(long cache_free_space, (const tec_event_cache_t *));
EXTERN_DECL(eif_boolean cache_get_curr_event,
            (tec_event_cache_t *, char **));
EXTERN_DECL(eif_boolean cache_step_file, (tec_event_cache_t *));
EXTERN_DECL(long cache_curr_step_no, (const tec_event_cache_t *));
EXTERN_DECL(eif_boolean cache_done_with_ev_cached_at_open,
            (const tec_event_cache_t *));
EXTERN_DECL(eif_boolean cache_append_event,
            (tec_event_cache_t *, const char*));
EXTERN_DECL(eif_boolean cache_sync, (tec_event_cache_t *,eif_boolean));
EXTERN_DECL(eif_boolean cache_truncate, (tec_event_cache_t *));
EXTERN_DECL(eif_boolean cache_dump,
            (const tec_event_cache_t*, cache_dump_func_t, void*,
             cache_dump_mode_t));


/*
 * Structure for the common part of a single connection.  Each type of
 * EIF (Managed Node, Endpoint, non-TME) includes this structure within
 * their specific tec_handle_t.
 */
typedef struct {
	AgentConfig	*agent_config;
	char			*location[TECEIF_MAX_SERVERS];
	FILE			*test_out;        /* Output stream for test mode               */
	int			max_locations;
	int			retry_window;

#ifndef NO_UTF8
	/* The local code set of the adapter - used for conversion to UTF-8 */
   tis_cs_t       adapter_local_cs_handle;

	/* The local code set of the TEC server - used for conversion from UTF-8
	 * in the event that we are sending to an OLD TEC server from a new adapter.
	 * This value is determined from the VERSION_THREE_SERVER_ENCODING variable
	 * in the adapter's conf file, and it is only needed in the case of
	 * connection-oriented TME managed node adapters or non-TME adapters. */
   tis_cs_t       tec_server_local_cs_handle;
#endif

	eif_boolean	in_flush_events;
	eif_boolean	test_mode;        /* Using test mode?                          */
	eif_boolean	dont_filter;      /* If set, tec_put_event() doesn't filter    */
	eif_boolean no_utf8_conversion;  /* If set, not convert to utf8 */

   tec_event_cache_t event_cache;

} tec_handle_common_t;

typedef struct {
	char	*source;
	int	id;
#if defined(WIN32)
	char	*log;
	char	*type;
#elif defined(NETWARE)
	int	locus;
	int	classname;
	int	severity;
#endif
} PreFilterEvent;

#define tec_disable_auto_filter(_h) ((tec_handle_common_t *)(_h))->dont_filter=1

/*
 * From tivoli/defines.h.  Copied here so that porting EIF is not
 * dependent on that TMF header (and the headers it includes).
 */
#ifndef E_RESERVED
#	define E_RESERVED 21
#endif

#ifndef E_HOST
#	define E_HOST	(E_RESERVED + 27) /* host inaccessible or host failure */
#endif

#define FILTER_KEEP             0
#define FILTER_DISCARD          1

/*
 * In agent_comm.c
 */
EXTERN_DECL(tec_handle_t nontme_tec_create_handle_r,
            (char*, char*, unsigned short, int, tec_delivery));
EXTERN_DECL(tec_handle_common_t* nontme_get_common_part, (tec_handle_t));
EXTERN_DECL(void nontme_tec_destroy_handle, (tec_handle_t));
EXTERN_DECL(long nontme_tec_put_event, (tec_handle_t, char*));
EXTERN_DECL(int nontme_tec_connection_status, (tec_handle_t));
EXTERN_DECL(void nontme_tec_handle_disconnect, (tec_handle_t));

/*
 * In tme_agent_comm.c and lcf_agent_comm.c
 */
EXTERN_DECL(tec_handle_t tme_tec_create_handle_r,
            (char*, char*, unsigned short, int, tec_delivery));
EXTERN_DECL(tec_handle_common_t* tme_get_common_part, (tec_handle_t));
EXTERN_DECL(void tme_tec_destroy_handle, (tec_handle_t));
EXTERN_DECL(long tme_tec_put_event, (tec_handle_t, char*));
EXTERN_DECL(int tme_tec_connection_status, (tec_handle_t));

/*
 * In tme_agent_comm.c
 */
EXTERN_DECL(int tec_gateway_init, (char *));
EXTERN_DECL(int tec_agent_get_retry, (tec_handle_t));
EXTERN_DECL(char* make_buff_file_name, (char *, char *));
typedef void (*TecAckCallback)(char*, long, void*);
EXTERN_DECL(void tec_set_ack_callback, (tec_handle_t, TecAckCallback, void*));
EXTERN_DECL(void tec_close_bdt_connection, (tec_handle_t));

/*
 * In agent_util.c
 */
/* only for internal use to check memory crash */
EXTERN_DECL(void fatal_memory_error, (VOID_DECL));
#ifndef NULL_CHECK
#define NULL_CHECK(p)     { if ((p) == NULL) fatal_memory_error(); }
#endif


EXTERN_DECL(int tec_buffer_event, (tec_handle_t, char *));
EXTERN_DECL(int tec_filter_event, (tec_handle_t, char *, int));
EXTERN_DECL(int tec_pre_filter_event, (tec_handle_t, PreFilterEvent *));
EXTERN_DECL(int common_agent_init, (char *cfgfile, AgentConfig **));
EXTERN_DECL(int common_agent_handle_init, (tec_handle_common_t *, char *, char*, char*));
EXTERN_DECL(int common_agent_get_retry, (tec_handle_common_t *));
EXTERN_DECL(int common_filter_event, (tec_handle_common_t *, char *event, int filter_type));
EXTERN_DECL(char *agent_getenv, (AgentConfig *, char *));
EXTERN_DECL(void tec_agent_setenv_r, (tec_handle_common_t*, char*, char*));
EXTERN_DECL(int agent_match, (AgentConfig *, char *, char *));
EXTERN_DECL(int common_agent_match, (tec_handle_common_t *, char *, char *));
EXTERN_DECL(char **common_split_entries_base, (char *, int *));
EXTERN_DECL(char **common_split_entries, (tec_handle_common_t *, char *, int *));
EXTERN_DECL(char *tec_normalize_msg, (char *, unsigned long));
EXTERN_DECL(int common_pre_filter_event, (tec_handle_common_t *, PreFilterEvent *));

#ifndef NO_UTF8
EXTERN_DECL(void tec_utf8_cleanup, (tec_handle_common_t *));
EXTERN_DECL(char *tec_prepend_utf8_byte_to_msg, (char *));
EXTERN_DECL(char *tec_to_utf8, (char *, tis_cs_t));
EXTERN_DECL(char *tec_to_server_encoding, (tec_handle_common_t *, char *));
EXTERN_DECL(char *tec_prep_utf8_msg, (tec_handle_common_t *, char *));
#endif

EXTERN_DECL(void gw_hack_turn_buffering_off, ());
EXTERN_DECL(void gw_hack_set_in_flush_events,(tec_handle_common_t *h));
extern int gw_hack_running_in_gateway;

/*
 * In evt_buffer.c
 */
EXTERN_DECL(void tec_buffer_init, (AgentConfig *));
EXTERN_DECL(int common_buffer_event, (tec_handle_common_t *, char *));
EXTERN_DECL(int common_buffer_event_to_file, (tec_handle_common_t *, char *, char *));
EXTERN_DECL(int common_buffer_event_to_file_hack, (tec_handle_common_t *, char *, char *, int));

EXTERN_DECL(int tec_flush_events_from_file, (tec_handle_t, char *));
#define tec_flush_events(_th) tec_flush_events_from_file(_th, NULL)

typedef int (*FlushCallback)(char *, void*);
EXTERN_DECL(int tec_flush_events_from_file2,
            (tec_handle_t, char *, FlushCallback, void*));

/*
 * Basic file-system stuff defined in evt_buffer.c.
 *
 * HISTORY: There was a bunch of #ifdef'd code in evt_buffer.c for dealing
 *    with basic file system stuff, and I moved it into a set up functions
 *    "sys_*()".  Unfortunately, we couldn't use file system functions from
 *    the framework because we can't assume the framework in the EIF.  I
 *    decided to go ahead an expose the functions here, but providing
 *    platform independent filesystem functions wasn't my task so the job
 *    isn't complete.  This kind of functionality ought to be put into a
 *    well-thought-out library some day.  See evt_buffer.c for usage.
 */
EXTERN_DECL(char * sys_basename, (const char*));
EXTERN_DECL(HANDLE sys_open_file, (const char*, int));
EXTERN_DECL(int sys_close_file, (HANDLE));
EXTERN_DECL(int sys_lock_file, (HANDLE));
EXTERN_DECL(int sys_unlock_file, (HANDLE));
EXTERN_DECL(int sys_get_file_size, (const char *, long *));
EXTERN_DECL(int sys_get_file_size2, (HANDLE, long *));
EXTERN_DECL(long sys_lseek, (HANDLE, long, int));
EXTERN_DECL(long sys_tell, (HANDLE));
EXTERN_DECL(long sys_read, (HANDLE, char *, long));
EXTERN_DECL(long sys_write, (HANDLE, const char *, long));
EXTERN_DECL(int sys_rename, (const char*, const char*));
EXTERN_DECL(int sys_remove, (const char*));
EXTERN_DECL(int sys_file_exists, (const char*));
EXTERN_DECL(int sys_truncate, (HANDLE, long));

#endif
