/*
 * Definitions etc. for regexp(3) routines.
 *
 * Caveat:  this is V8 regexp(3) [actually, a reimplementation thereof],
 * not the System V one.
 * 
 * Syntax of Regular Expressions:
 * ------------------------------
 * 
 * Regular expressions are implemented using Henry Spencer's package
 * (thanks, Henry!), and much of the description of regular expressions
 * below is copied verbatim from his manual entry.
 * 
 * A regular expression is zero or more branches, separated by ``|''. It
 * matches anything that matches one of the branches.
 * 
 * A branch is zero or more pieces, concatenated. It matches a match for
 * the first, followed by a match for the second, etc.
 * 
 * A piece is an atom possibly followed by ``*'', ``+'', or ``?''. An
 * atom followed by ``*'' matches a sequence of 0 or more matches of the
 * atom. An atom followed by ``+'' matches a sequence of 1 or more
 * matches of the atom. An atom followed by ``?'' matches a match of the
 * atom, or the null string.
 * 
 * An atom is a regular expression in parentheses (matching a match for
 * the regular expression), a range (see below), ``.'' (matching any
 * single character), ``^'' (matching the null string at the beginning of
 * the input string), ``$'' (matching the null string at the end of the
 * input string), a ``\'' followed by a single character (matching that
 * character), or a single character with no other significance (matching
 * that character).
 * 
 * A range is a sequence of characters enclosed in ``[]''. It normally
 * matches any single character from the sequence. If the sequence begins
 * with ``^'', it matches any single character not from the rest of the
 * sequence. If two characters in the sequence are separated by ``-'',
 * this is shorthand for the full list of ASCII characters between them
 * (e.g. ``[0-9]'' matches any decimal digit). To include a literal ``]''
 * in the sequence, make it the first character (following a possible
 * ``^''). To include a literal ``-'', make it the first or last
 * character.
 * 
 */

#ifndef _REGEXP
#define _REGEXP 1

#define _ANSI_ARGS_(x) x
#define CONST          const

#define NSUBEXP  10
typedef struct regexp {
	char *startp[NSUBEXP];
	char *endp[NSUBEXP];
	char regstart;		/* Internal use only. */
	char reganch;		/* Internal use only. */
	char *regmust;		/* Internal use only. */
	int regmlen;		/* Internal use only. */
	char program[1];	/* Unwarranted chumminess with compiler. */
} regexp;

typedef void (*RegErrHandler)(char*);
extern RegErrHandler regErrHandlerFunc;

/* NOTE - 09/05/2000 - Renamed regcomp/regexec to eif_regcomp/eif_regexec
 *   to avoid conflicts w/ functions of the same name in framework libraries.
 */
extern regexp *eif_regcomp _ANSI_ARGS_((char *exp));
extern int eif_regexec _ANSI_ARGS_((regexp *prog, char *string));

#endif /* REGEXP */
