/*****************************************************************************/
/*                                                                           */
/*  FILE: slist.h                                                            */
/*                                                                           */
/*  FUNCTION: Header file for T/EC Adapter structured list utilities.        */
/*                                                                           */
/*****************************************************************************/

#ifndef _SLIST_H
#define _SLIST_H

/**********************************/
/*** Constant Macro Definitions ***/
/**********************************/

#define BLOCK_SIZE	5

#ifndef TRUE
#	define TRUE 1
#	define FALSE 0
#endif

/************************/
/*** Enumerated types ***/
/************************/

/***********************/
/*** Function Macros ***/
/***********************/

/************************************/
/*** Class/Structure Definitions  ***/
/************************************/

struct SL {
	void	**list;
	int	nbr;
	int	last_pos;
};

typedef struct {
	void	*list;
	int	nbr;
	int	last_pos;
	int	elt_size;
} SA_t;

/****************/
/*** Typedefs ***/
/****************/

typedef struct SL SL_t;

typedef int		(*SL_compare_fn) (void *, void *);
typedef void	(*SL_free_fn) (void *);
typedef int		(*SL_walk_fn) (void *, int);
typedef int		(*SL_walk2_fn) (void *, void *);
typedef int		(*SL_walkva_fn) ();

typedef SL_walk2_fn	SA_walk2_fn;

#define SL_GetPos(_lh, _p)	(((_p >= _lh->last_pos) || (_p < 0)) ? NULL : (_lh->list[_p]))
#define SL_NbrElm(_lh)		((_lh != NULL) ? _lh->last_pos : 0)
#define SL_Create()			((SL_t *) calloc(1, sizeof(SL_t)))

/**********************/
/*** Global Externs ***/
/**********************/

/***************************/
/*** Function Prototypes ***/
/***************************/

extern int	SL_InsLast     (SL_t *, void *);
extern void	*SL_Find       (SL_t *, void *, SL_compare_fn);
extern int	SL_FindPos     (SL_t *, void *, SL_compare_fn);
extern int	SL_FindReplace (SL_t *, void *, SL_compare_fn, SL_free_fn);
extern int	SL_Delete      (SL_t *, void *, SL_free_fn);
extern void	SL_DelAll      (SL_t *, SL_free_fn);
extern int	SL_Walk        (SL_t *, SL_walk_fn);
extern int	SL_Walk2       (SL_t *, SL_walk2_fn, void *);

#if !defined(__ILEC400__) /* function not currently supported */
   #if defined(__STDC__) || defined(STDC_SRC_COMPATIBLE)
	extern int SL_WalkVa    (SL_t *, SL_walkva_fn, ...);
   #else
	extern int SL_WalkVa    ();
   #endif
#endif

#define SA_NbrElm(_ah)		((_ah != NULL) ? _ah->last_pos : 0)

extern SA_t	*SA_Create		(int);
extern void	SA_Delete		(SA_t *, void *);
extern void	*SA_InsLast		(SA_t *, void *);
extern int	SA_Walk2			(SA_t *, SA_walk2_fn, void *);

#endif 

