/*
 * Component Name: EIF
 *
 * $Date: 1996/12/09 23:07:57 $
 *
 * $Source:
 *
 * $Revision: 1.35 $
 *
 * Description:
 *      TEC agent communication library for non-TME agents
 *
 * (C) COPYRIGHT TIVOLI Systems, Inc. 1994
 * Unpublished Work
 * All Rights Reserved
 * Licensed Material - Property of TIVOLI Systems, Inc.
 */


#include <arpa/inet.h>
#include <unistd.h>

#include <rpc/rpc.h>
#include <rpc/pmap_prot.h>
#include <rpc/pmap_clnt.h>


static struct timeval timeout = { 0, 50000 };
static struct timeval tottimeout = { 0, 50000 };

/*
 * These are defined in agent_util.c and allow you to change the timeout
 * times that this function waits for a port.
 */
extern time_t  GETPORT_TIMEOUT_SECONDS;
extern long    GETPORT_TIMEOUT_USEC;
extern time_t  GETPORT_TOTAL_TIMEOUT_SECONDS;
extern long    GETPORT_TOTAL_TIMEOUT_USEC;

/*
 * Find the mapped port for program,version.
 * Calls the pmap service remotely to do the lookup.
 * Returns 0 if no map exists.
 */
u_short
tec_pmap_getport(address, program, version, protocol)
	struct sockaddr_in *address;
	u_long program;
	u_long version;
	u_int protocol;
{
	u_short port = 0;
	int socket = -1;
	register CLIENT *client;
	struct pmap parms;

	/* IY25233 - These timeout times may be set in the *.conf file.
	 * They are the same names except in lower case (see agent_util.c).
	 */
	timeout.tv_sec = GETPORT_TIMEOUT_SECONDS;
	timeout.tv_usec = GETPORT_TIMEOUT_USEC;
	tottimeout.tv_sec = GETPORT_TOTAL_TIMEOUT_SECONDS;
	tottimeout.tv_usec = GETPORT_TOTAL_TIMEOUT_USEC;

	address->sin_port = htons(PMAPPORT);
	client = clntudp_bufcreate(address, PMAPPROG,
	    PMAPVERS, timeout, &socket, RPCSMALLMSGSIZE, RPCSMALLMSGSIZE);
	if (client != (CLIENT *)NULL) {
		parms.pm_prog = program;
		parms.pm_vers = version;
		parms.pm_prot = protocol;
		parms.pm_port = 0;  /* not needed or used */
		if (CLNT_CALL(client, PMAPPROC_GETPORT, xdr_pmap, &parms,
		    xdr_u_short, &port, tottimeout) != RPC_SUCCESS){
			rpc_createerr.cf_stat = RPC_PMAPFAILURE;
			clnt_geterr(client, &rpc_createerr.cf_error);
		} else if (port == 0) {
			rpc_createerr.cf_stat = RPC_PROGNOTREGISTERED;
		}
		CLNT_DESTROY(client);
	}
	(void)close(socket);

	address->sin_port = 0;
	return (port);
}
